package org.jboss.security.jacc;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.security.Permission;
import java.security.PermissionCollection;
import java.security.Permissions;
import java.security.Principal;
import java.security.ProtectionDomain;
import java.security.acl.Group;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import javax.security.jacc.PolicyContextException;
import org.jboss.logging.Logger;

public class ContextPolicy {
  private static Logger log = Logger.getLogger(org.jboss.security.jacc.ContextPolicy.class);
  
  private String contextID;
  
  private Permissions excludedPermissions = new Permissions();
  
  private Permissions uncheckedPermissions = new Permissions();
  
  private HashMap rolePermissions = new HashMap();
  
  private boolean trace;
  
  ContextPolicy(String contextID) {
    this.contextID = contextID;
    this.trace = log.isTraceEnabled();
  }
  
  Permissions getPermissions() {
    Permissions perms = new Permissions();
    Enumeration eter = this.uncheckedPermissions.elements();
    while (eter.hasMoreElements()) {
      Permission p = eter.nextElement();
      perms.add(p);
    } 
    Iterator iter = this.rolePermissions.values().iterator();
    while (iter.hasNext()) {
      Permissions rp = iter.next();
      eter = rp.elements();
      while (eter.hasMoreElements()) {
        Permission p = eter.nextElement();
        perms.add(p);
      } 
    } 
    return perms;
  }
  
  boolean implies(ProtectionDomain domain, Permission permission) {
    boolean implied = false;
    if (this.excludedPermissions.implies(permission)) {
      if (this.trace)
        log.trace("Denied: Matched excluded set, permission=" + permission); 
      return false;
    } 
    if (this.uncheckedPermissions.implies(permission)) {
      if (this.trace)
        log.trace("Allowed: Matched unchecked set, permission=" + permission); 
      return true;
    } 
    Principal[] principals = domain.getPrincipals();
    int length = (principals != null) ? principals.length : 0;
    ArrayList princpalNames = new ArrayList();
    int n;
    for (n = 0; n < length; n++) {
      Principal p = principals[n];
      if (p instanceof Group) {
        Group g = (Group)p;
        Enumeration iter = g.members();
        while (iter.hasMoreElements()) {
          p = iter.nextElement();
          String name = p.getName();
          princpalNames.add(name);
        } 
      } else {
        String name = p.getName();
        princpalNames.add(name);
      } 
    } 
    if (princpalNames.size() > 0) {
      for (n = 0; !implied && n < princpalNames.size(); n++) {
        String name = princpalNames.get(n);
        Permissions perms = (Permissions)this.rolePermissions.get(name);
        if (this.trace)
          log.trace("Checking role=" + name + " perms=" + perms); 
        if (perms != null) {
          implied = perms.implies(permission);
          if (this.trace)
            log.trace((implied ? "Allowed: " : "Denied: ") + " permission=" + permission); 
        } 
      } 
    } else if (this.trace) {
      log.trace("No principals found in domain: " + domain);
    } 
    return implied;
  }
  
  void clear() {
    this.excludedPermissions = new Permissions();
    this.uncheckedPermissions = new Permissions();
    this.rolePermissions.clear();
  }
  
  void addToExcludedPolicy(Permission permission) throws PolicyContextException {
    this.excludedPermissions.add(permission);
  }
  
  void addToExcludedPolicy(PermissionCollection permissions) throws PolicyContextException {
    Enumeration iter = permissions.elements();
    while (iter.hasMoreElements()) {
      Permission p = iter.nextElement();
      this.excludedPermissions.add(p);
    } 
  }
  
  void addToRole(String roleName, Permission permission) throws PolicyContextException {
    Permissions perms = (Permissions)this.rolePermissions.get(roleName);
    if (perms == null) {
      perms = new Permissions();
      this.rolePermissions.put(roleName, perms);
    } 
    perms.add(permission);
  }
  
  void addToRole(String roleName, PermissionCollection permissions) throws PolicyContextException {
    Permissions perms = (Permissions)this.rolePermissions.get(roleName);
    if (perms == null) {
      perms = new Permissions();
      this.rolePermissions.put(roleName, perms);
    } 
    Enumeration iter = permissions.elements();
    while (iter.hasMoreElements()) {
      Permission p = iter.nextElement();
      perms.add(p);
    } 
  }
  
  void addToUncheckedPolicy(Permission permission) throws PolicyContextException {
    this.uncheckedPermissions.add(permission);
  }
  
  void addToUncheckedPolicy(PermissionCollection permissions) throws PolicyContextException {
    Enumeration iter = permissions.elements();
    while (iter.hasMoreElements()) {
      Permission p = iter.nextElement();
      this.uncheckedPermissions.add(p);
    } 
  }
  
  void commit() throws PolicyContextException {}
  
  void delete() throws PolicyContextException {
    clear();
  }
  
  String getContextID() throws PolicyContextException {
    return this.contextID;
  }
  
  void linkConfiguration(org.jboss.security.jacc.ContextPolicy link) throws PolicyContextException {}
  
  void removeExcludedPolicy() throws PolicyContextException {
    this.excludedPermissions = new Permissions();
  }
  
  void removeRole(String roleName) throws PolicyContextException {
    this.rolePermissions.remove(roleName);
  }
  
  void removeUncheckedPolicy() throws PolicyContextException {
    this.uncheckedPermissions = new Permissions();
  }
  
  public String toString() {
    StringBuffer tmp = new StringBuffer("<ContextPolicy contextID='");
    tmp.append(this.contextID);
    tmp.append("'>\n");
    tmp.append("\t<ExcludedPermissions>\n");
    Enumeration iter = this.excludedPermissions.elements();
    while (iter.hasMoreElements()) {
      Permission p = iter.nextElement();
      tmp.append("<Permission type='");
      tmp.append(p.getClass());
      tmp.append("' name='");
      tmp.append(p.getName());
      tmp.append("' actions='");
      tmp.append(p.getActions());
      tmp.append("' />\n");
    } 
    tmp.append("\t</ExcludedPermissions>\n");
    tmp.append("\t<UncheckedPermissions>\n");
    iter = this.uncheckedPermissions.elements();
    while (iter.hasMoreElements()) {
      Permission p = iter.nextElement();
      tmp.append("<Permission type='");
      tmp.append(p.getClass());
      tmp.append(" name='");
      tmp.append(p.getName());
      tmp.append("' actions='");
      tmp.append(p.getActions());
      tmp.append("' />\n");
    } 
    tmp.append("\t</UncheckedPermissions>\n");
    tmp.append("\t<RolePermssions>\n");
    Iterator roles = this.rolePermissions.keySet().iterator();
    while (roles.hasNext()) {
      String role = roles.next();
      Permissions perms = (Permissions)this.rolePermissions.get(role);
      iter = perms.elements();
      tmp.append("\t\t<Role name='" + role + "'>\n");
      while (iter.hasMoreElements()) {
        Permission p = iter.nextElement();
        tmp.append("<Permission type='");
        tmp.append(p.getClass());
        tmp.append(" name='");
        tmp.append(p.getName());
        tmp.append("' actions='");
        tmp.append(p.getActions());
        tmp.append("' />\n");
      } 
      tmp.append("\t\t</Role>\n");
    } 
    tmp.append("\t</RolePermssions>");
    tmp.append("</ContextPolicy>\n");
    return tmp.toString();
  }
}
